﻿using System;
using System.Windows.Navigation;
using Microsoft.Devices.Sensors;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;

namespace MotionMatrix
{
    public partial class MainPage : PhoneApplicationPage
    {
        Motion motion;

        public MainPage()
        {
            InitializeComponent();

            if (Motion.IsSupported)
            {
                motion = new Motion();
                motion.TimeBetweenUpdates = TimeSpan.FromMilliseconds(33);
                motion.CurrentValueChanged += OnMotionCurrentValueChanged;
            }
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            // Start the Motion sensor if it's available
            if (motion != null)
            {
                try { motion.Start(); }
                catch { }
            }
            base.OnNavigatedTo(args);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            if (motion != null)
            {
                motion.Stop();
            }
            base.OnNavigatedFrom(args);
        }

        void OnMotionCurrentValueChanged(object sender, SensorReadingEventArgs<MotionReading> args)
        {
            this.Dispatcher.BeginInvoke(() =>
                {
                    AttitudeReading attitude = args.SensorReading.Attitude;

                    pitchText.Text = (180 * attitude.Pitch / Math.PI).ToString("F0");
                    rollText.Text = (180 * attitude.Roll / Math.PI).ToString("F0");
                    yawText.Text = (180 * attitude.Yaw / Math.PI).ToString("F0");

                    Matrix matrix = attitude.RotationMatrix;

                    m11Text.Text = matrix.M11.ToString("F3");
                    m12Text.Text = matrix.M12.ToString("F3");
                    m13Text.Text = matrix.M13.ToString("F3");

                    m21Text.Text = matrix.M21.ToString("F3");
                    m22Text.Text = matrix.M22.ToString("F3");
                    m23Text.Text = matrix.M23.ToString("F3");

                    m31Text.Text = matrix.M31.ToString("F3");
                    m32Text.Text = matrix.M32.ToString("F3");
                    m33Text.Text = matrix.M33.ToString("F3");

                    double angle = Math.Acos((matrix.M11 + matrix.M22 + matrix.M33 - 1) / 2);
                    angleText.Text = (180 * angle / Math.PI).ToString("F0");

                    if (angle != 0)
                    {
                        double twoSine = 2 * Math.Sin(angle);
                        double x = (matrix.M23 - matrix.M32) / twoSine;
                        double y = (matrix.M31 - matrix.M13) / twoSine;
                        double z = (matrix.M12 - matrix.M21) / twoSine;

                        axisText.Text = String.Format("({0:F2} {1:F2} {2:F2})", x, y, z);
                    }
                });
        }
    }
}