using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Devices.Sensors;

namespace View3DPhoneCameraMotion
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        Model model;
        Matrix scaleMatrix, worldMatrix, lookatMatrix,
               viewMatrix, projectionMatrix;
        Motion motion;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            TargetElapsedTime = TimeSpan.FromTicks(333333);
            InactiveSleepTime = TimeSpan.FromSeconds(1);

            // Switch to portrait
            graphics.IsFullScreen = true;
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;

            // Create Motion object
            if (Motion.IsSupported)
            {
                motion = new Motion
                {
                    TimeBetweenUpdates = this.TargetElapsedTime
                };
            }
        }

        protected override void OnActivated(object sender, EventArgs args)
        {
            if (motion != null)
            {
                try { motion.Start(); }
                catch { }
            }
            base.OnActivated(sender, args);
        }

        protected override void OnDeactivated(object sender, EventArgs args)
        {
            if (motion != null)
                motion.Stop();

            base.OnDeactivated(sender, args);
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            model = this.Content.Load<Model>("Phone Retro Caesar N170910");

            foreach (ModelMesh mesh in model.Meshes)
                foreach (BasicEffect effect in mesh.Effects)
                    effect.EnableDefaultLighting();

            // Scale matrix
            scaleMatrix = Matrix.CreateScale(0.0001f);
            worldMatrix = scaleMatrix;

            // View matrix
            lookatMatrix = Matrix.CreateLookAt(new Vector3(0, 0, 10f),
                                               Vector3.Zero,
                                               Vector3.Up);
            viewMatrix = lookatMatrix;

            // Projection matrix
            float aspectRatio = graphics.GraphicsDevice.Viewport.AspectRatio;
            projectionMatrix =
                Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
                                                    aspectRatio,
                                                    1f, 100.0f);
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            if (motion != null && motion.IsDataValid)
                viewMatrix = Matrix.CreateRotationX(MathHelper.PiOver2) *
                                motion.CurrentValue.Attitude.RotationMatrix *
                                lookatMatrix;
            else
                viewMatrix = Matrix.CreateRotationX(MathHelper.PiOver2) *
                                lookatMatrix;

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            model.Draw(worldMatrix, viewMatrix, projectionMatrix);
            base.Draw(gameTime);
        }
    }
}
