using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace View3DPhoneReoriented
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        Model model;
        Matrix scaleMatrix, worldMatrix, lookatMatrix,
               viewMatrix, projectionMatrix;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            TargetElapsedTime = TimeSpan.FromTicks(333333);
            InactiveSleepTime = TimeSpan.FromSeconds(1);

            // Switch to portrait
            graphics.IsFullScreen = true;
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            model = this.Content.Load<Model>("Phone Retro Caesar N170910");

            foreach (ModelMesh mesh in model.Meshes)
                foreach (BasicEffect effect in mesh.Effects)
                    effect.EnableDefaultLighting();

            // World matrix
            scaleMatrix = Matrix.CreateScale(0.0001f);
            worldMatrix = scaleMatrix;
            
            // Rotate so looking at top
            worldMatrix *= Matrix.CreateRotationX(MathHelper.PiOver2); 

            // View matrix
            lookatMatrix = Matrix.CreateLookAt(new Vector3(0, 0, 10f),
                                               Vector3.Zero,
                                               Vector3.Up);
            viewMatrix = lookatMatrix;

            // Projection matrix
            float aspectRatio = graphics.GraphicsDevice.Viewport.AspectRatio;
            projectionMatrix =
                Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
                                                    aspectRatio,
                                                    1f, 100.0f);
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            model.Draw(worldMatrix, viewMatrix, projectionMatrix);
            base.Draw(gameTime);
        }
    }
}
