﻿using System;
using System.Windows;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Devices.Sensors;

namespace YawPitchRoll
{
    public partial class MainPage : PhoneApplicationPage
    {
        Motion motion;

        public MainPage()
        {
            InitializeComponent();

            if (Motion.IsSupported)
            {
                motion = new Motion();
                motion.TimeBetweenUpdates = TimeSpan.FromMilliseconds(33);
                motion.CurrentValueChanged += OnMotionCurrentValueChanged;
                motion.Calibrate += OnMotionCalibrate;
            }
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            if (motion != null)
            {
                try { motion.Start(); }
                catch { }
            }

            base.OnNavigatedTo(args);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            if (motion != null)
                motion.Stop();

            base.OnNavigatedFrom(e);
        }

        void OnMotionCalibrate(object sender, CalibrationEventArgs args)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                (this.Resources["calibrateStoryboard"] as Storyboard).Begin();
            });
        }

        void OnMotionCurrentValueChanged(object sender, SensorReadingEventArgs<MotionReading> args)
        {
            AttitudeReading attitude = args.SensorReading.Attitude;

            this.Dispatcher.BeginInvoke(() =>
                {
                    yawValue.Text = (180 * attitude.Yaw / Math.PI).ToString("F0") + "°";
                    pitchValue.Text = (180 * attitude.Pitch / Math.PI).ToString("F0") + "°";
                    rollValue.Text = (180 * attitude.Roll / Math.PI).ToString("F0") + "°";

                    yawRotate.Angle = 180 * attitude.Yaw / Math.PI;

                    if (attitude.Pitch <= Math.PI / 2 && attitude.Pitch >= -Math.PI / 2)
                    {
                        pitchPath.Fill = pitchPath.Stroke;
                        pitchEllipse.Center = new Point(ContentPanel.ActualWidth / 2,
                                                        ContentPanel.ActualHeight * (attitude.Pitch + Math.PI / 2) / Math.PI);
                    }
                    else
                    {
                        pitchPath.Fill = null;

                        if (attitude.Pitch > Math.PI / 2)
                            pitchEllipse.Center = new Point(ContentPanel.ActualWidth / 2,
                                                            ContentPanel.ActualHeight * (3 * Math.PI / 2 - attitude.Pitch) / Math.PI);
                        else
                            pitchEllipse.Center = new Point(ContentPanel.ActualWidth / 2,
                                                            ContentPanel.ActualHeight * (-Math.PI / 2 - attitude.Pitch) / Math.PI);
                    }

                    rollEllipse.Center = new Point(ContentPanel.ActualWidth * (attitude.Roll + Math.PI / 2) / Math.PI,
                                                   ContentPanel.ActualHeight / 2);
                });
        }
    }
}